-----------------------------------------------------------------------
-- Get local references to certain often-used global objects for better speed
-----------------------------------------------------------------------
local _G = getfenv(0)

local math = _G.math
local table = _G.table

local ipairs = _G.ipairs
local pairs = _G.pairs
local setmetatable = _G.setmetatable
local tonumber = _G.tonumber
local tostring = _G.tostring
local unpack = _G.unpack

-----------------------------------------------------------------------
-- WoW passes in the addon name and a private table, so grab those
-----------------------------------------------------------------------
local ADDON_NAME, private = ...

local Jughead = LibStub("AceAddon-3.0"):NewAddon("Jughead", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0" ) -- , "AceHook-3.0", "AceBucket-3.0"
Jughead.version = _G.GetAddOnMetadata(ADDON_NAME, "Version")

local Astrolabe = _G.DongleStub("Astrolabe-1.0")

-- Allow other people to find me.. if for some reason they want to anyway
_G["Jughead"] = Jughead
local Archy = _G["Archy"]

local L = LibStub("AceLocale-3.0"):GetLocale("Jughead", true)

Jughead.enabled = true

SURVEY_GREEN = 1
SURVEY_YELLOW = 2
SURVEY_RED = 3


-----------------------------------------------------------------------
-- "Prototype" some functions so we can use them
-----------------------------------------------------------------------
local dumpMinimapStats
local fetchArchySurveyPOI

-----------------------------------------------------------------------
-- Some variables for keeping track of things
-----------------------------------------------------------------------
local current_continent  -- Where the heck am I anyway?  Updated by timer
local timer_handle -- used to keep track of player location
local race_data = {}  -- used to keep track of how many fragments we have of each race, so we can know when to turn POI frames back on.
local myButtonFrame   -- holds my frame of red/yellow/green/reset buttons
local SURVEY_YARDS = {[SURVEY_GREEN] = 40, [SURVEY_YELLOW] = 80, [SURVEY_RED] = 800}
local SURVEY_ANGLES = {[SURVEY_GREEN] = {left=math.rad(20),right=math.rad(340)}, [SURVEY_YELLOW] = {left=math.rad(30),right=math.rad(330)}, [SURVEY_RED] = {left=math.rad(45),right=math.rad(315)}}
local archDigFrame -- The Archy dig site frame


----------------------------------------
----------------------------------------
local function fetchArchySurveyPOI()
	
	local kids = { Minimap:GetChildren() }
	local archyKids = {}

	for _, cFrame in pairs(kids) do
		local fName = cFrame:GetName()
		
		if fName and fName:match("ArchyMinimap_SurveyPOI%d+") then
			if cFrame.type == "survey" then
				table.insert(archyKids,cFrame)
			end
		end

	end
	
	return archyKids

end


----------------------------------------
----------------------------------------
local function fetchGM2SurveyPOI()
	
	local kids = { Minimap:GetChildren() }
	local gm2Kids = {}

	for _, cFrame in pairs(kids) do
		local fName = cFrame:GetName()
		
		if fName and fName:match("GatherMatePin%d+") then
			if cFrame.type == "survey" then
				table.insert(gm2Kids,cFrame)
			end
		end

	end
	
	return gm2Kids

end



----------------------------------------
----------------------------------------
local function dumpMinimapStats()

	local kids = fetchArchySurveyPOI()

	for _, cFrame in pairs(kids) do
		cFrame.icon:Hide()
		cFrame:Hide()
	end

	Jughead:Print( "Found " .. #kids .. " Archy survey frames.")
	
end




----------------------------------------
----------------------------------------
local function fetchArchyDigSiteFrame()
	
	local kids = { _G.UIParent:GetChildren() }
	local retval
	
	-- Jughead:Print( "Looking for ArchyDigSiteFrame.  Found " .. #kids .. " child frames" )
	
	for _, cFrame in pairs(kids) do
		local fName = cFrame:GetName()
		
		if fName and fName == "ArchyDigSiteFrame" then
			-- Jughead:Print( "Found ArchyDigSiteFrame" )
			retval = cFrame
			break
		end
	end
	
	return retval

end


----------------------------------------
----------------------------------------
local function enableAllArchySurveyPOI()
	local kids = fetchArchySurveyPOI()

	for _, cFrame in pairs(kids) do
		cFrame.icon:Show()
		cFrame:Show()
	end
end



----------------------------------------
-- minDist  POI closer to player than minDist will be hidden
-- maxDist  POI farther from player than maxDist will be hidden
----------------------------------------
local function hideArchySurveyPOI(minDist, maxDist, minAngle, maxAngle)

	local kids = fetchArchySurveyPOI()

	Jughead:Print( "minDist=" .. minDist .. " maxDist=" .. maxDist .. " minAngle=" .. minAngle .. " maxAngle=" .. maxAngle )	

	local playerFacing = _G.GetPlayerFacing()
	local rad_360 = math.rad(360)  -- use this to normalize resulting angle
	
	Jughead:Print( "GetPlayerFacing=" .. playerFacing .. " rad_360=" .. rad_360 )	

	for _, cFrame in pairs(kids) do
		-- This gives the angle, starting from North, moving CClkwise from the player point (extended to North) to the given icon
		local angle = Astrolabe:GetDirectionToIcon( cFrame )
		if angle then
			-- To adjust for angle RELATIVE TO PLAYER, we need to SUBTRACT the player facing 
			angle = angle - playerFacing
			if angle < 0 then
				angle = angle + rad_360
			end
	
			Jughead:Print( cFrame:GetName() .. " adjusted angle=" .. angle )	
	
			local showIt = false
			
			if angle <= minAngle or angle >= maxAngle then
				-- ok, it's in the right set of angles, now look for distance
				
				local dist = Astrolabe:GetDistanceToIcon( cFrame )
				
				Jughead:Print( cFrame:GetName() .. " distance=" .. dist )	
				
				if dist >= minDist and dist <= maxDist then
					showIt = true
				end
			
			
			end
			
			if showIt then
				cFrame.icon:Show()
				cFrame:Show()
			else
				cFrame.icon:Hide()
				cFrame:Hide()
			end				
		
			-- Jughead:Print( cFrame:GetName() .. "  " .. angle )		
		end
	end


end

----------------------------------------
----------------------------------------
function buttonRed_OnLClick()
	hideArchySurveyPOI( SURVEY_YARDS[SURVEY_YELLOW], SURVEY_YARDS[SURVEY_RED], SURVEY_ANGLES[SURVEY_RED].left, SURVEY_ANGLES[SURVEY_RED].right )
end


----------------------------------------
----------------------------------------
function buttonYellow_OnLClick()
	hideArchySurveyPOI( SURVEY_YARDS[SURVEY_GREEN], SURVEY_YARDS[SURVEY_YELLOW], SURVEY_ANGLES[SURVEY_YELLOW].left, SURVEY_ANGLES[SURVEY_YELLOW].right )
end

----------------------------------------
----------------------------------------
function buttonGreen_OnLClick()
	hideArchySurveyPOI( 0, SURVEY_YARDS[SURVEY_GREEN], SURVEY_ANGLES[SURVEY_GREEN].left, SURVEY_ANGLES[SURVEY_GREEN].right )
end

----------------------------------------
----------------------------------------
function buttonReset_OnLClick()

	enableAllArchySurveyPOI()
	
end

-----------------------------------------------------------------------
--
-- Event handlers and timers
--
-----------------------------------------------------------------------

-----------------------------------------------------------------------
-- Mouse events
-----------------------------------------------------------------------

----------------------------------------
----------------------------------------
function Jughead_ButtonRed_OnMouseDown(self, button)
--	if button == "LeftButton" then
		buttonRed_OnLClick()
--	elseif button == "RightButton" then
--		buttonRed_OnRClick()
--	end
end

----------------------------------------
----------------------------------------
function Jughead_ButtonYellow_OnMouseDown(self, button)
	buttonYellow_OnLClick()
end

----------------------------------------
----------------------------------------
function Jughead_ButtonGreen_OnMouseDown(self, button)
	buttonGreen_OnLClick()
end

----------------------------------------
----------------------------------------
function Jughead_ButtonReset_OnMouseDown(self, button)
	buttonReset_OnLClick()
end


-----------------------------------------------------------------------
-- Timers
-----------------------------------------------------------------------

----------------------------------------
-- Called by the timer we set up in PLAYER_ENTERING_WORLD event
----------------------------------------
function Jughead:UpdatePlayerPosition(force)
	current_continent = _G.GetCurrentMapContinent()
end


-----------------------------------------------------------------------
-- Game events
-----------------------------------------------------------------------

----------------------------------------
-- When we enter the world, start keeping track of where the player is
----------------------------------------
function Jughead:PLAYER_ENTERING_WORLD()
	-- Two timers are needed here: If we force a call to UpdatePlayerPosition() too soon, the site distances will not update properly and the notifications may vanish just as the player is able to see them.
	if not timer_handle and self.enabled then
		self:ScheduleTimer(function()
			timer_handle = self:ScheduleRepeatingTimer("UpdatePlayerPosition", 5)
		end, 2)
	end
--	self:ScheduleTimer("UpdatePlayerPosition", 2, true)

end

----------------------------------------
----------------------------------------
function Jughead:CURRENCY_DISPLAY_UPDATE()
	if not current_continent or _G.GetNumArchaeologyRaces() == 0 or not self.enabled then
		return
	end

	for race_id = 1, _G.GetNumArchaeologyRaces() do
		local _, _, _, currencyAmount = _G.GetArchaeologyRaceInfo(race_id)
		local diff = currencyAmount - (race_data[race_id].currency or 0)

		race_data[race_id].currency = currencyAmount

		if diff < 0 then
			-- we've "spent" fragments by solving an artifact.. nothing to do here, move along..
		elseif diff > 0 then
			-- we've gained fragments so we have successfully dug.  Turn the survey nodes back on

			enableAllArchySurveyPOI()

		end
	end
end


-- for index, node in pairs(Archy.db.global.surveyNodes[site.id]) do


-----------------------------------------------------------------------
--
-- Addon events
--
-----------------------------------------------------------------------

----------------------------------------
----------------------------------------
function Jughead:OnInitialize()

	if not _G["Archy"] then
		Jughead:Print( "Archy isn't here.  I'm going to the malt shop to find him." )
		self.enabled = false
	else
		Jughead:Print( "I'm hanging out with Archy.")
	end
end

----------------------------------------
----------------------------------------
function Jughead:OnEnable()
	-- Called when the addon is enabled
	if self.enabled then

		Jughead:Print( ADDON_NAME .. " " .. Jughead.version .. " " .. Archy.version )

		Jughead:RegisterChatCommand("jughead", "MySlashProcessorFunc")

		self:RegisterEvent("PLAYER_ENTERING_WORLD")
		self:RegisterEvent("CURRENCY_DISPLAY_UPDATE")

		-- Archy frames might not be created at this point yet, so delay a bit..		
		self:ScheduleTimer(function()
			-- Find the ArchyDistanceIndicator frame
			local archDigFrame = fetchArchyDigSiteFrame()

			-- Now, attach our frame to it		
			if archDigFrame then
				--Jughead:Print( "Found Archy's dig frame.")
				myButtonFrame = _G.CreateFrame("Frame", "JugheadDistanceAttachmentFrame", archDigFrame, "JugheadDistanceAttachment")
				myButtonFrame:Show()
			end
		end, 5)

	end
end

----------------------------------------
----------------------------------------
function Jughead:OnDisable()
    -- Called when the addon is disabled
    if self.enabled then
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
		self:UnregisterEvent("CURRENCY_DISPLAY_UPDATE")
    	
    	if myButtonFrame then
    		myButtonFrame:Hide()
    	end
    end
    
    if timer_handle then
	    self:CancelTimer(timer_handle)
	end
    
end


----------------------------------------
-- Slash command handler
----------------------------------------
function Jughead:MySlashProcessorFunc(input)
  -- Process the slash command ('input' contains whatever follows the slash command)
    if self.enabled then
		local command = input:lower()

		if command == L["dump"]:lower() then
			dumpMinimapStats()
		end
    
    end
end



